#!/usr/bin/env bash
#
# ARG_OPTIONAL_SINGLE([path],[p],[Path to the new package. Will be created if it does not exist],[.])
# ARG_OPTIONAL_SINGLE([name],[n],[Name of the created package],[my-cool-package])
# ARG_OPTIONAL_SINGLE([initial-version],[],[The initial version of the package],[0.0.0])
# ARG_OPTIONAL_SINGLE([description],[d],[Description text for the package],[""])
# ARG_OPTIONAL_SINGLE([author-name],[a],[Name of the author],["$(git config --global user.name)"])
# ARG_OPTIONAL_SINGLE([author-email],[e],[Email of the author],["$(git config --global user.email)"])
# ARG_OPTIONAL_SINGLE([license],[],[License for the new package],["MIT"])
# ARG_OPTIONAL_SINGLE([type],[t],[What type of package do you want to create],["library"])
# ARG_TYPE_GROUP_SET([operations],[OPERATION],[type],[library,application],[index])
# ARG_OPTIONAL_SINGLE([git-origin],[g],[The url of the git origin (like "git@github.com:Zebreus/create-typescript-thing.git")],[""])
# ARG_OPTIONAL_SINGLE([git-main-branch],[],[The name of the man git branch],["master"])
# ARG_OPTIONAL_BOOLEAN([in-monorepo],[],[If the package is part of a monorepo. Will not try to create a git repo if set.])
# ARG_OPTIONAL_BOOLEAN([dry-run],[],[Do nothing])
# ARG_HELP([Create a new typescript project])
# ARGBASH_GO()
# needed because of Argbash --> m4_ignore([
### START OF CODE GENERATED BY Argbash v2.10.0 one line above ###
# Argbash is a bash code generator used to get arguments parsing right.
# Argbash is FREE SOFTWARE, see https://argbash.io for more info


die()
{
	local _ret="${2:-1}"
	test "${_PRINT_HELP:-no}" = yes && print_help >&2
	echo "$1" >&2
	exit "${_ret}"
}

# validators

operations()
{
	local _allowed=("library" "application") _seeking="$1" _idx=0
	for element in "${_allowed[@]}"
	do
		test "$element" = "$_seeking" && { test "$3" = "idx" && echo "$_idx" || echo "$element"; } && return 0
		_idx=$((_idx + 1))
	done
	die "Value '$_seeking' (of argument '$2') doesn't match the list of allowed values: 'library' and 'application'" 4
}


begins_with_short_option()
{
	local first_option all_short_options='pndaetgh'
	first_option="${1:0:1}"
	test "$all_short_options" = "${all_short_options/$first_option/}" && return 1 || return 0
}

# THE DEFAULTS INITIALIZATION - OPTIONALS
_arg_path="."
_arg_name="my-cool-package"
_arg_initial_version="0.0.0"
_arg_description=""
_arg_author_name="$(git config --global user.name)"
_arg_author_email="$(git config --global user.email)"
_arg_license="MIT"
_arg_type="library"
_arg_git_origin=""
_arg_git_main_branch="master"
_arg_in_monorepo="off"
_arg_dry_run="off"


print_help()
{
	printf '%s\n' "Create a new typescript project"
	printf 'Usage: %s [-p|--path <arg>] [-n|--name <arg>] [--initial-version <arg>] [-d|--description <arg>] [-a|--author-name <arg>] [-e|--author-email <arg>] [--license <arg>] [-t|--type <OPERATION>] [-g|--git-origin <arg>] [--git-main-branch <arg>] [--(no-)in-monorepo] [--(no-)dry-run] [-h|--help]\n' "$0"
	printf '\t%s\n' "-p, --path: Path to the new package. Will be created if it does not exist (default: '.')"
	printf '\t%s\n' "-n, --name: Name of the created package (default: 'my-cool-package')"
	printf '\t%s\n' "--initial-version: The initial version of the package (default: '0.0.0')"
	printf '\t%s\n' "-d, --description: Description text for the package (default: '""')"
	printf '\t%s\n' "-a, --author-name: Name of the author (default: '"$(git config --global user.name)"')"
	printf '\t%s\n' "-e, --author-email: Email of the author (default: '"$(git config --global user.email)"')"
	printf '\t%s\n' "--license: License for the new package (default: '"MIT"')"
	printf '\t%s\n' "-t, --type: What type of package do you want to create. Can be one of: 'library' and 'application' (default: '"library"')"
	printf '\t%s\n' "-g, --git-origin: The url of the git origin (like \"git@github.com:Zebreus/create-typescript-thing.git\") (default: '""')"
	printf '\t%s\n' "--git-main-branch: The name of the man git branch (default: '"master"')"
	printf '\t%s\n' "--in-monorepo, --no-in-monorepo: If the package is part of a monorepo. Will not try to create a git repo if set. (off by default)"
	printf '\t%s\n' "--dry-run, --no-dry-run: Do nothing (off by default)"
	printf '\t%s\n' "-h, --help: Prints help"
}


parse_commandline()
{
	while test $# -gt 0
	do
		_key="$1"
		case "$_key" in
			-p|--path)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_path="$2"
				shift
				;;
			--path=*)
				_arg_path="${_key##--path=}"
				;;
			-p*)
				_arg_path="${_key##-p}"
				;;
			-n|--name)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_name="$2"
				shift
				;;
			--name=*)
				_arg_name="${_key##--name=}"
				;;
			-n*)
				_arg_name="${_key##-n}"
				;;
			--initial-version)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_initial_version="$2"
				shift
				;;
			--initial-version=*)
				_arg_initial_version="${_key##--initial-version=}"
				;;
			-d|--description)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_description="$2"
				shift
				;;
			--description=*)
				_arg_description="${_key##--description=}"
				;;
			-d*)
				_arg_description="${_key##-d}"
				;;
			-a|--author-name)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_author_name="$2"
				shift
				;;
			--author-name=*)
				_arg_author_name="${_key##--author-name=}"
				;;
			-a*)
				_arg_author_name="${_key##-a}"
				;;
			-e|--author-email)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_author_email="$2"
				shift
				;;
			--author-email=*)
				_arg_author_email="${_key##--author-email=}"
				;;
			-e*)
				_arg_author_email="${_key##-e}"
				;;
			--license)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_license="$2"
				shift
				;;
			--license=*)
				_arg_license="${_key##--license=}"
				;;
			-t|--type)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_type="$(operations "$2" "type")" || exit 1
				shift
				;;
			--type=*)
				_arg_type="$(operations "${_key##--type=}" "type")" || exit 1
				;;
			-t*)
				_arg_type="$(operations "${_key##-t}" "type")" || exit 1
				;;
			-g|--git-origin)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_git_origin="$2"
				shift
				;;
			--git-origin=*)
				_arg_git_origin="${_key##--git-origin=}"
				;;
			-g*)
				_arg_git_origin="${_key##-g}"
				;;
			--git-main-branch)
				test $# -lt 2 && die "Missing value for the optional argument '$_key'." 1
				_arg_git_main_branch="$2"
				shift
				;;
			--git-main-branch=*)
				_arg_git_main_branch="${_key##--git-main-branch=}"
				;;
			--no-in-monorepo|--in-monorepo)
				_arg_in_monorepo="on"
				test "${1:0:5}" = "--no-" && _arg_in_monorepo="off"
				;;
			--no-dry-run|--dry-run)
				_arg_dry_run="on"
				test "${1:0:5}" = "--no-" && _arg_dry_run="off"
				;;
			-h|--help)
				print_help
				exit 0
				;;
			-h*)
				print_help
				exit 0
				;;
			*)
				_PRINT_HELP=yes die "FATAL ERROR: Got an unexpected argument '$1'" 1
				;;
		esac
		shift
	done
}

parse_commandline "$@"

# OTHER STUFF GENERATED BY Argbash
# Validation of values
_arg_type_index="$(operations "$_arg_type" "type" idx)"

### END OF CODE GENERATED BY Argbash (sortof) ### ])
# [ <-- needed because of Argbash


set +e

PROJECT_PATH="$_arg_path"
NAME="$_arg_name"
VERSION="$_arg_initial_version"
DESCRIPTION="$_arg_description"
AUTHOR_NAME="$_arg_author_name"
AUTHOR_EMAIL="$_arg_author_email"
LICENSE="$_arg_license"

TYPE="$_arg_type"
# TYPE="library"

# True if the project is the root of a git repo, false if the project is nested in another git repo.
GIT_ROOT="$(test "$_arg_in_monorepo" = "on" && echo "false" || echo "true")"
GIT_ORIGIN="$_arg_git_origin"
GIT_MAIN_BRANCH="$_arg_git_main_branch"

SCRIPTPATH="$( cd -- "$(dirname "$0")" >/dev/null 2>&1 ; pwd -P )"

# resource="$1"
# target="$2"
cp_resource_file() { 
if test "$1" = ".prettierrc.js" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/.prettierrc.js"
const organizeImports = require("prettier-plugin-organize-imports")

module.exports = {
  arrowParens: "avoid",
  bracketSpacing: true,
  embeddedLanguageFormatting: "auto",
  htmlWhitespaceSensitivity: "css",
  insertPragma: false,
  jsxBracketSameLine: false,
  jsxSingleQuote: false,
  printWidth: 120,
  proseWrap: "always",
  quoteProps: "consistent",
  requirePragma: false,
  semi: false,
  singleQuote: false,
  tabWidth: 2,
  trailingComma: "es5",
  useTabs: false,
  vueIndentScriptAndStyle: false,
  plugins: [organizeImports],
}
EOF_SUFFIX
cp "resources_tmp/.prettierrc.js" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "shell.nix" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/shell.nix"
{ pkgs ? import <nixpkgs> { } }:
with pkgs;
mkShell {
  nativeBuildInputs = [
    nodejs-18_x
    yarn
    git
    jq
    moreutils
  ];
  shellHook = with pkgs; ''
    export PATH="$(pwd)/node_modules/.bin:$PATH"
  '';
}

EOF_SUFFIX
cp "resources_tmp/shell.nix" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "jest.config.js" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/jest.config.js"
module.exports = {
  roots: ["<rootDir>/src"],
  moduleDirectories: ["node_modules", "src"],
  transform: {
    "^.+\\.tsx?$": "ts-jest",
  },
  coverageThreshold: {
    global: {
      statements: 95,
    },
  },
};

EOF_SUFFIX
cp "resources_tmp/jest.config.js" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "applicationIndex.ts" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/applicationIndex.ts"
#!/usr/bin/env node

console.log("You launched the application!")

export {}
EOF_SUFFIX
cp "resources_tmp/applicationIndex.ts" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "example.test.ts" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/example.test.ts"
describe("some examples work", () => {
  test("one is one", async () => {
    expect(1).toEqual(1);
  });

  test("two is two", async () => {
    expect(2).toEqual(2);
  });

  test("throw throws", async () => {
    expect(() => {throw new Error("eyyy")}).toThrow();
  })
});

export {}
EOF_SUFFIX
cp "resources_tmp/example.test.ts" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = ".eslintrc.json" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/.eslintrc.json"
{
  "env": {
    "browser": true,
    "node": true,
    "es6": true
  },
  "extends": [
    "eslint:recommended",
    "plugin:@typescript-eslint/eslint-recommended",
    "plugin:@typescript-eslint/recommended"
  ],
  "plugins": [
    "import",
    "@typescript-eslint"
  ],
  "rules": {
    "@typescript-eslint/explicit-module-boundary-types": "off",
    "@typescript-eslint/no-unused-vars": [
      "error",
      {
        "ignoreRestSiblings": true
      }
    ],
    "@typescript-eslint/switch-exhaustiveness-check": "error",
    "@typescript-eslint/no-empty-function": "off",
    "no-undef": "off",
    "no-unused-vars": [
      "off",
      {
        "ignoreRestSiblings": true
      }
    ],
    "import/first": "error",
    "import/no-duplicates": [
      "error",
      {
        "considerQueryString": true
      }
    ],
    "import/no-namespace": "error",
    "import/extensions": [
      "error",
      "always",
      {
        "js": "never",
        "jsx": "never",
        "ts": "never",
        "tsx": "never"
      }
    ],
    "import/order": [
      "error",
      {
        "groups": [
          [
            "internal",
            "external",
            "builtin"
          ],
          [
            "parent",
            "sibling",
            "index"
          ]
        ],
        "alphabetize": {
          "order": "asc",
          "caseInsensitive": true
        }
      }
    ],
    "import/newline-after-import": "error",
    "import/no-anonymous-default-export": "error",
    "import/no-unresolved": "off",
    "import/named": "off",
    "import/default": "error",
    "import/namespace": "error",
    "import/no-dynamic-require": "error",
    "import/no-self-import": "error",
    "import/no-useless-path-segments": [
      "error",
      {
        "noUselessIndex": true
      }
    ],
    "import/no-relative-packages": "error",
    "import/export": "error",
    "import/no-unused-modules": "error",
    "import/no-deprecated": "error",
    "import/no-named-as-default-member": "warn"
  },
  "overrides": [
    {
      "files": [
        "*.ts",
        "*.tsx"
      ],
      "parserOptions": {
        "parser": "@typescript-eslint/parser",
        "ecmaVersion": 2018,
        "sourceType": "module",
        "project": "./tsconfig.json"
      },
      "settings": {
        "import/external-module-folders": [
          "node_modules",
          "node_modules/@types"
        ],
        "import/parsers": {
          "@typescript-eslint/parser": [
            ".ts",
            ".tsx"
          ]
        },
        "import/resolver": {
          "node": {
            "extensions": []
          }
        }
      }
    }
  ]
}
EOF_SUFFIX
cp "resources_tmp/.eslintrc.json" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "libraryIndex.ts" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/libraryIndex.ts"
export const add = (a: number, b: number) => a + b;
export const subtract = (a: number, b: number) => a + b;
EOF_SUFFIX
cp "resources_tmp/libraryIndex.ts" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "settings.json" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/settings.json"
{
  "editor.defaultFormatter": "esbenp.prettier-vscode",
  "[typescriptreact]": {
    "editor.defaultFormatter": "esbenp.prettier-vscode"
  },
  "[typescript]": {
    "editor.defaultFormatter": "esbenp.prettier-vscode"
  },
  "[javascriptreact]": {
    "editor.defaultFormatter": "esbenp.prettier-vscode"
  },
  "[javascript]": {
    "editor.defaultFormatter": "esbenp.prettier-vscode"
  },
  "[jsonc]": {
    "editor.defaultFormatter": "esbenp.prettier-vscode"
  },
  "[json]": {
    "editor.defaultFormatter": "esbenp.prettier-vscode"
  },
  "editor.formatOnSave": true,
  "editor.codeActionsOnSave": {
    "source.fixAll.eslint": true
  },
  "gitlens.hovers.currentLine.over": "line",
  "editor.tabCompletion": "on",
  "editor.inlineSuggest.enabled": true,
  "typescript.updateImportsOnFileMove.enabled": "always",
  "eslint.packageManager": "yarn",
  "editor.quickSuggestions": {
    "strings": true
  }
}
EOF_SUFFIX
cp "resources_tmp/settings.json" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "extensions.json" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/extensions.json"
{
  "recommendations": [
    "dbaeumer.vscode-eslint",
    "eamodio.gitlens",
    "wix.vscode-import-cost",
    "orta.vscode-jest",
    "esbenp.prettier-vscode"
  ]
}

EOF_SUFFIX
cp "resources_tmp/extensions.json" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = ".lintstagedrc.json" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/.lintstagedrc.json"
{
    "*.+(ts|tsx)": [
        "prettier --write",
        "eslint --cache --fix",
        "tsc-files --noEmit"
    ],
    "*.+(js|jsx)": [
        "prettier --write",
        "eslint --cache --fix"
    ],
    "*.+(json|css|md|yml|yaml|scss)": [
        "prettier --write"
    ]
}
EOF_SUFFIX
cp "resources_tmp/.lintstagedrc.json" "$2"
rm -rf resources_tmp
return 0
fi
if test "$1" = "tsconfig.json" ; then
mkdir -p resources_tmp
cat <<\EOF_SUFFIX > "resources_tmp/tsconfig.json"
{
  "compilerOptions": {
    "incremental": true,
    "target": "es2016",
    "lib": [
      "es2016",
      "esnext"
    ],
    "jsx": "preserve",
    "module": "commonjs",
    "rootDir": "src",
    "baseUrl": "./src/",
    "allowJs": true,
    "declaration": true,
    "declarationMap": true,
    "sourceMap": true,
    "outDir": "./dist",
    "noEmit": true,
    "isolatedModules": true,
    "allowSyntheticDefaultImports": true,
    "esModuleInterop": true,
    "forceConsistentCasingInFileNames": true,
    "strict": true,
    "skipLibCheck": true
  },
  "include": [
    "src"
  ]
}
EOF_SUFFIX
cp "resources_tmp/tsconfig.json" "$2"
rm -rf resources_tmp
return 0
fi
echo "Failed to find file $1 in resources" ; exit 1
}

jqi() {
    file=$1
    query=$2
    cat $file | jq --indent 2 "$query" | sponge $file
}

add_script() {
    name=$1
    content=$2
    cat package.json | jq --indent 2  '.scripts.'"$name"'="'"$content"'"' | sponge package.json
}

append_script() {
    name=$1
    content=$2
    jqi package.json ".scripts.$name = if .scripts.$name then (.scripts.$name + \" && \" + \"$content\") else \"$content\" end"
}

add_ignore() {
    file=$1
    value=$2
    label=$3
    touch "$file"
    comment="# $label"
    comment_line="$(grep -Fxn "$comment" "$file" | cut -f1 -d: | head -n1)"
    if test -z "$comment_line"; then
        echo "$comment" >> "$file"
        echo "$value" >> "$file"
        echo "" >> "$file"
        return 0
    fi
    first_content_line=$(expr $comment_line + 1)
    content_length_maybe="$(tail "$file" -n +"$first_content_line" | grep -Pn "^(#.*)?$"  | cut -f1 -d: | head -n 1)"
    content_length="$(expr $(test -n "$content_length_maybe" && echo "$content_length_maybe" || echo 99999 ) - 1)"
    if tail "$file" -n +"$first_content_line" | head -n"$content_length" | grep -Fxn "$value" > /dev/null ; then
      # Already exists
      return 0
    fi
    insert_line=$(expr $first_content_line + $content_length)
    sed -i "$insert_line i $value" "$file"
}

add_gitignore() {
    add_ignore .gitignore "$1" "$2"
}

mkdir -p $PROJECT_PATH
cd $PROJECT_PATH

# Abort on dry run
if test "$_arg_dry_run" = "on" ; then
    echo "Dry run, aborting"
    exit 0
fi

# Make sure we are inside a git repo
if $GIT_ROOT && ! test -e .git
then
    git init -b "$GIT_MAIN_BRANCH"
    if test -n "$GIT_ORIGIN"
    then
        git remote add origin "$GIT_ORIGIN"
        git pull -f --set-upstream origin "$GIT_MAIN_BRANCH"
    fi
else
    if test -n "$GIT_ORIGIN" && test "$(git remote get-url origin)" != "$GIT_ORIGIN"
    then
        echo "You specified an origin for your git repo, but the project is already a git repo with a different origin." >&2
        exit 1
    fi
fi

if ! git rev-parse --is-inside-work-tree
then
    echo "The project has to be inside a nested git repo, if GIT_ROOT is false" >&2
    exit 1
fi

# Ensure that package.json does not already exist
if test -e package.json
then
    echo "package.json already exists" >&2
    exit 1
fi

# Add nix-shell
cp_resource_file shell.nix .
git add .
git commit -m "Add nix shell"

# Initialize project
yarn init --yes
test -z "$VERSION" || jqi package.json ".version = \"$VERSION\""
test -z "$DESCRIPTION" || jqi package.json ".description = \"$DESCRIPTION\""
jqi package.json "del(.author)"
test -z "$AUTHOR_NAME" || jqi package.json ".author.name = \"$AUTHOR_NAME\""
test -z "$AUTHOR_EMAIL" || jqi package.json ".author.email = \"$AUTHOR_EMAIL\""
jqi package.json "del(.repository)"
if ! $GIT_ROOT
then
    GIT_ORIGIN="$(git remote get-url origin)"
    jqi package.json ".repository.directory = \"$(git rev-parse --show-prefix)\""
fi
if test -n "$GIT_ORIGIN"
then
    jqi package.json ".repository.type = \"git\""
    if echo "$GIT_ORIGIN" | grep -Po "^https://"
    then
        jqi package.json ".repository.url = \"$GIT_ORIGIN\""
    elif echo "$GIT_ORIGIN" | grep -Po "^git@"
    then
        value="$(echo "$GIT_ORIGIN" | sed 's/:/\//' | sed 's/^git@/https:\/\//' | sed 's/.git$//')"
        jqi package.json ".repository.url = \"$value\""
    else
        jqi package.json ".repository.url = \"$GIT_ORIGIN\""
    fi
fi
jqi package.json ".license = \"$LICENSE\""
add_gitignore "node_modules" "node"
git add .
git commit -m "Initialize node project"

# Add typescript
yarn add --dev typescript@latest @types/node@latest
cp_resource_file tsconfig.json .
mkdir -p src
add_gitignore dist/ typescript
add_gitignore '*.tsbuildinfo' typescript
add_script "build" "tsc"
git add .
git commit -m "Install typescript"

# Add prettier
yarn add --dev prettier@latest prettier-plugin-organize-imports
cp_resource_file .prettierrc.js .
add_script "format" "prettier --write ."
git add .
git commit -m "Install prettier"

# Add eslint
yarn add --dev eslint @types/eslint @typescript-eslint/eslint-plugin@latest @typescript-eslint/parser@latest eslint-plugin-import@latest
cp_resource_file .eslintrc.json .
add_script "lint" "eslint --cache && tsc --noEmit"
add_gitignore ".eslintcache" "eslint"
git add .
git commit -m "Install eslint"

# Add jest
yarn add --dev jest@latest @types/jest@latest ts-jest@latest ts-node@latest eslint-plugin-jest@latest
cp_resource_file jest.config.js .
mkdir -p src/tests
cp_resource_file example.test.ts src/tests/example.test.ts
jqi tsconfig.json '.exclude |= (.+ ["src/tests"] | unique)'
jqi .eslintrc.json '.extends |= (.+ ["plugin:jest/recommended"] | unique)'
jqi .eslintrc.json '.rules["jest/expect-expect"] = "off"'
add_script "test" "jest"
git add .
git commit -m "Install jest"

# Add lint-staged
yarn add --dev lint-staged@latest tsc-files@latest
cp_resource_file .lintstagedrc.json .
git add .
git commit -m "Install lint-staged"

# Add husky
if $GIT_ROOT
then
  yarn add --dev husky@latest pinst@latest
  yarn husky install
  yarn husky add .husky/pre-commit "FORCE_COLOR=1 yarn lint-staged"
  add_script "postinstall" "husky install"
  add_script "prepack" "pinst --disable"
  add_script "postpack" "pinst --enable"
  git add .
  git commit -m "Install pre-commit hook"
fi

# Add vscode presets
mkdir -p .vscode
cp_resource_file extensions.json ./.vscode
cp_resource_file settings.json ./.vscode
git add .
git commit -m "Add vscode extensions and settings"

# Configure library project
if test "$TYPE" = "library"
then
  yarn add --dev @vercel/ncc@latest
  jqi package.json '.files |= (.+ ["dist/**"] | unique)'
  jqi package.json '.keywords |= (.+ ["library"] | unique)'
  jqi package.json '.main = "dist/index.js"'
  add_script "build" "ncc build --out dist --minify src/index.ts"
  append_script "prepack" "ncc build --out dist --minify src/index.ts"
  append_script "prepublish" "eslint --cache && tsc --noEmit"
  mkdir -p src
  cp_resource_file libraryIndex.ts src/index.ts
  git add .
  git commit -m "Configured project as library"
fi

# Configure executable project
if test "$TYPE" = "application"
then
  yarn add --dev @vercel/ncc@latest
  jqi package.json '.files |= (.+ ["dist/**"] | unique)'
  jqi package.json '.keywords |= (.+ ["executable", "application", "bin"] | unique)'
  jqi package.json '.main = "dist/index.js"'
  jqi package.json ".bin[\"$NAME\"] = \"dist/index.js\""
  add_script "build" "ncc build --out dist --minify src/index.ts"
  append_script "prepack" "ncc build --out dist --minify src/index.ts"
  append_script "prepublish" "eslint --cache && tsc --noEmit"
  add_script "start" "ncc run src/index.ts"
  mkdir -p src
  cp_resource_file applicationIndex.ts src/index.ts
  git add .
  git commit -m "Configured project as application"
fi

# ] <-- needed because of Argbash
